@@ -1,7 +1,9 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import (
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
 import sys
 import os
@@ -17,7 +19,6 @@ import pwd
 import grp
 import time
 
-logging.basicConfig() # clufter needs logging set before imported
 try:
     import clufter.format_manager
     import clufter.filter_manager
@@ -26,15 +27,25 @@ try:
 except ImportError:
     no_clufter = True
 
-import settings
-import utils
-import cluster
-import constraint
-import prop
-import resource
-import status
-import stonith
-import usage
+from pcs import (
+    cluster,
+    constraint,
+    prop,
+    quorum,
+    resource,
+    settings,
+    status,
+    stonith,
+    usage,
+    utils,
+    alert,
+)
+from pcs.lib.errors import LibraryError
+from pcs.lib.commands import quorum as lib_quorum
+import pcs.cli.constraint_colocation.command as colocation_command
+import pcs.cli.constraint_order.command as order_command
+import pcs.cli.constraint_ticket.command as ticket_command
+from pcs.cli.common.console_report import indent
 
 
 def config_cmd(argv):
@@ -84,6 +95,14 @@ def config_show(argv):
     print()
     config_show_cib()
     cluster.cluster_uidgid([], True)
+    if "--corosync_conf" in utils.pcs_options or not utils.is_rhel6():
+        print()
+        print("Quorum:")
+        try:
+            config = lib_quorum.get_config(utils.get_lib_env())
+            print("\n".join(indent(quorum.quorum_config_to_str(config))))
+        except LibraryError as e:
+            utils.process_library_reports(e.args)
 
 def config_show_cib():
     print("Resources:")
@@ -96,9 +115,17 @@ def config_show_cib():
     print("Fencing Levels:")
     print()
     stonith.stonith_level_show()
+
+    lib = utils.get_library_wrapper()
     constraint.location_show([])
-    constraint.order_show([])
-    constraint.colocation_show([])
+    modificators = utils.get_modificators()
+    order_command.show(lib, [], modificators)
+    colocation_command.show(lib, [], modificators)
+    ticket_command.show(lib, [], modificators)
+
+    print()
+    alert.print_alert_config(lib, [], modificators)
+
     print()
     del utils.pcs_options["--all"]
     print("Resources Defaults:")
@@ -240,7 +267,7 @@ def config_restore_remote(infile_name, infile_obj):
                     % node
                 )
                 continue
-        except (ValueError, NameError):
+        except (ValueError, NameError, LookupError):
             err_msgs.append("unable to determine status of the node %s" % node)
     if err_msgs:
         for msg in err_msgs:
