@@ -1,22 +1,23 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import (
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
-import os.path
-import sys
 import shutil
 import unittest
 import xml.dom.minidom
-parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-sys.path.insert(0, parentdir)
 
-from pcs_test_functions import pcs, ac
-import rule
+from pcs import rule
+from pcs.test.tools.misc import (
+    ac,
+    get_test_resource as rc,
+)
+from pcs.test.tools.pcs_runner import pcs
 
-
-empty_cib = "empty.xml"
-temp_cib = "temp.xml"
+empty_cib = rc("cib-empty.xml")
+temp_cib = rc("temp-cib.xml")
 
 class DateValueTest(unittest.TestCase):
 
@@ -1242,7 +1243,7 @@ class CibBuilderTest(unittest.TestCase):
         )
 
     def assertExpressionXml(self, rule_expression, rule_xml):
-        cib_dom = xml.dom.minidom.parse("empty.xml")
+        cib_dom = xml.dom.minidom.parse(empty_cib)
         constraints = cib_dom.getElementsByTagName("constraints")[0]
         constraint_el = constraints.appendChild(
             cib_dom.createElement("rsc_location")
@@ -1779,14 +1780,14 @@ Location Constraints:
       Rule: score=INFINITY  (id:location-dummy1-rule)
         Expression: #uname eq node1  (id:location-dummy1-rule-expr)
     Constraint: location-dummy1-1
-      Rule: score=100 role=master  (id:MyRule)
+      Rule: role=master score=100  (id:MyRule)
         Expression: #uname eq node2  (id:MyRule-expr)
     Constraint: location-dummy1-2
-      Rule: score=INFINITY boolean-op=or  (id:complexRule)
-        Rule: score=0 boolean-op=and  (id:complexRule-rule)
+      Rule: boolean-op=or score=INFINITY  (id:complexRule)
+        Rule: boolean-op=and score=0  (id:complexRule-rule)
           Expression: #uname eq node3  (id:complexRule-rule-expr)
           Expression: foo gt version 1.2  (id:complexRule-rule-expr-1)
-        Rule: score=0 boolean-op=and  (id:complexRule-rule-1)
+        Rule: boolean-op=and score=0  (id:complexRule-rule-1)
           Expression:  (id:complexRule-rule-1-expr)
             Date Spec: hours=12-23 weekdays=1-5  (id:complexRule-rule-1-expr-datespec)
           Expression: date in_range 2014-07-26 to duration  (id:complexRule-rule-1-expr-1)
@@ -1802,14 +1803,14 @@ Location Constraints:
       Rule: score=INFINITY
         Expression: #uname eq node1
     Constraint: location-dummy1-1
-      Rule: score=100 role=master
+      Rule: role=master score=100
         Expression: #uname eq node2
     Constraint: location-dummy1-2
-      Rule: score=INFINITY boolean-op=or
-        Rule: score=0 boolean-op=and
+      Rule: boolean-op=or score=INFINITY
+        Rule: boolean-op=and score=0
           Expression: #uname eq node3
           Expression: foo gt version 1.2
-        Rule: score=0 boolean-op=and
+        Rule: boolean-op=and score=0
           Expression:
             Date Spec: hours=12-23 weekdays=1-5
           Expression: date in_range 2014-07-26 to duration
@@ -1927,7 +1928,7 @@ Location Constraints:
         self.assertEqual(1, returnVal)
 
     def assertExpressionXml(self, rule_expression, rule_xml):
-        cib_dom = xml.dom.minidom.parse("empty.xml")
+        cib_dom = xml.dom.minidom.parse(empty_cib)
         constraints = cib_dom.getElementsByTagName("constraints")[0]
         constraint_el = constraints.appendChild(
             cib_dom.createElement("rsc_location")
@@ -1939,8 +1940,3 @@ Location Constraints:
             constraint_el.toprettyxml(indent="    "),
             rule_xml.lstrip().rstrip(" ")
         )
-
-
-if __name__ == "__main__":
-    unittest.main()
-
