@@ -16,6 +16,9 @@ require 'resource.rb'
 require 'cluster_entity.rb'
 require 'auth.rb'
 
+class NotImplementedException < NotImplementedError
+end
+
 def getAllSettings(auth_user, cib_dom=nil)
   unless cib_dom
     cib_dom = get_cib_dom(auth_user)
@@ -128,6 +131,40 @@ def add_order_set_constraint(
   return retval, stderr.join(' ')
 end
 
+
+def add_ticket_constraint(
+    auth_user, ticket, resource_id, role, loss_policy,
+    force=false, autocorrect=true
+)
+  command = [PCS, "constraint", "ticket", "add", ticket]
+  if role
+    command << role
+  end
+  command << resource_id
+  command << 'loss-policy=' + loss_policy unless loss_policy.strip().empty?()
+  command << '--force' if force
+  command << '--autocorrect' if autocorrect
+  stdout, stderr, retval = run_cmd(auth_user, *command)
+  return retval, stderr.join(' ')
+end
+
+def add_ticket_set_constraint(
+  auth_user, ticket, loss_policy, resource_set_list, force=false,
+  autocorrect=true
+)
+  command = [PCS, 'constraint', 'ticket']
+  resource_set_list.each { |resource_set|
+    command << 'set'
+    command.concat(resource_set)
+  }
+  command << 'setoptions'
+  command << 'ticket=' + ticket
+  command << 'loss-policy=' + loss_policy unless loss_policy.strip().empty?()
+  command << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, *command)
+  return retval, stderr.join(' ')
+end
+
 def add_colocation_constraint(
   auth_user, resourceA, resourceB, score, force=false, autocorrect=true
 )
@@ -242,7 +279,7 @@ end
 
 # Gets all of the nodes specified in the pcs config file for the cluster
 def get_cluster_nodes(cluster_name)
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   clusters = pcs_config.clusters
   cluster = nil
   for c in clusters
@@ -405,9 +442,12 @@ def send_request(auth_user, node, request, post=false, data={}, remote=true, raw
   end
 end
 
-def add_node(auth_user, new_nodename, all=false, auto_start=true)
+def add_node(auth_user, new_nodename, all=false, auto_start=true, watchdog=nil)
   if all
     command = [PCS, "cluster", "node", "add", new_nodename]
+    if watchdog and not watchdog.strip.empty?
+      command << "--watchdog=#{watchdog.strip}"
+    end
     if auto_start
       command << '--start'
       command << '--enable'
@@ -420,7 +460,7 @@ def add_node(auth_user, new_nodename, all=false, auto_start=true)
   end
   $logger.info("Adding #{new_nodename} to pcs_settings.conf")
   corosync_nodes = get_corosync_nodes()
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   pcs_config.update_cluster($cluster_name, corosync_nodes)
   sync_config = Cfgsync::PcsdSettings.from_text(pcs_config.text())
   # on version conflict just go on, config will be corrected eventually
@@ -444,7 +484,7 @@ def remove_node(auth_user, new_nodename, all=false)
   end
   $logger.info("Removing #{new_nodename} from pcs_settings.conf")
   corosync_nodes = get_corosync_nodes()
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   pcs_config.update_cluster($cluster_name, corosync_nodes)
   sync_config = Cfgsync::PcsdSettings.from_text(pcs_config.text())
   # on version conflict just go on, config will be corrected eventually
@@ -818,24 +858,6 @@ def disable_cluster(auth_user)
   return true
 end
 
-def corosync_running?()
-  if ISSYSTEMCTL
-    `systemctl status corosync.service`
-  else
-    `service corosync status`
-  end
-  return $?.success?
-end
-
-def corosync_enabled?()
-  if ISSYSTEMCTL
-    `systemctl is-enabled corosync.service`
-  else
-    `chkconfig corosync`
-  end
-  return $?.success?
-end
-
 def get_corosync_version()
   begin
     stdout, stderror, retval = run_cmd(
@@ -854,21 +876,7 @@ def get_corosync_version()
 end
 
 def pacemaker_running?()
-  if ISSYSTEMCTL
-    `systemctl status pacemaker.service`
-  else
-    `service pacemaker status`
-  end
-  return $?.success?
-end
-
-def pacemaker_enabled?()
-  if ISSYSTEMCTL
-    `systemctl is-enabled pacemaker.service`
-  else
-    `chkconfig pacemaker`
-  end
-  return $?.success?
+  is_service_running?('pacemaker')
 end
 
 def get_pacemaker_version()
@@ -888,15 +896,6 @@ def get_pacemaker_version()
   return nil
 end
 
-def cman_running?()
-  if ISSYSTEMCTL
-    `systemctl status cman.service`
-  else
-    `service cman status`
-  end
-  return $?.success?
-end
-
 def get_cman_version()
   begin
     stdout, stderror, retval = run_cmd(
@@ -936,15 +935,6 @@ def pcsd_restart()
   }
 end
 
-def pcsd_enabled?()
-  if ISSYSTEMCTL
-    `systemctl is-enabled pcsd.service`
-  else
-    `chkconfig pcsd`
-  end
-  return $?.success?
-end
-
 def get_pcsd_version()
   return PCS_VERSION.split(".").collect { | x | x.to_i }
 end
@@ -1003,12 +993,12 @@ def is_cib_true(var)
 end
 
 def read_tokens()
-  return PCSTokens.new(Cfgsync::PcsdTokens.from_file('').text()).tokens
+  return PCSTokens.new(Cfgsync::PcsdTokens.from_file().text()).tokens
 end
 
 def write_tokens(tokens)
   begin
-    cfg = PCSTokens.new(Cfgsync::PcsdTokens.from_file('').text())
+    cfg = PCSTokens.new(Cfgsync::PcsdTokens.from_file().text())
     cfg.tokens = tokens
     Cfgsync::PcsdTokens.from_text(cfg.text()).save()
   rescue
@@ -1124,7 +1114,7 @@ def pcs_auth(auth_user, nodes, username, password, force=false, local=true)
   }
   if not new_tokens.empty?
     cluster_nodes = get_corosync_nodes()
-    tokens_cfg = Cfgsync::PcsdTokens.from_file('')
+    tokens_cfg = Cfgsync::PcsdTokens.from_file()
     # only tokens used in pcsd-to-pcsd communication can and need to be synced
     # those are accessible only when running under root account
     if Process.uid != 0
@@ -1425,6 +1415,7 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     :status => 'unknown',
     :node_list => [],
     :resource_list => [],
+    :available_features => [],
   }
 
   threads = []
@@ -1453,6 +1444,7 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
       }
       begin
         parsed_response = JSON.parse(response, {:symbolize_names => true})
+        parsed_response[:available_features] ||= []
         if parsed_response[:noresponse]
           node_map[node][:node] = {}
           node_map[node][:node].update(node_status_unknown)
@@ -1537,6 +1529,31 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     }
   end
   status.delete(:node)
+  sbd_enabled = []
+  sbd_running = []
+  sbd_disabled_node_list = []
+  node_map.each { |_, cluster_status|
+    node_status = cluster_status[:node][:status]
+    node_name = cluster_status[:node][:name]
+    # create set of available features on all nodes
+    # it is intersection of available features from all nodes
+    if node_status != 'unknown' and cluster_status[:available_features]
+      status[:available_features] &= cluster_status[:available_features]
+    end
+    if (
+      cluster_status[:node][:services] and
+      cluster_status[:node][:services][:sbd]
+    )
+      if cluster_status[:node][:services][:sbd][:enabled]
+        sbd_enabled << node_name
+      else
+        sbd_disabled_node_list << node_name if node_status != 'unknown'
+      end
+      if cluster_status[:node][:services][:sbd][:running]
+        sbd_running << node_name
+      end
+    end
+  }
 
   if status[:quorate]
     fence_count = 0
@@ -1545,9 +1562,9 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
         fence_count += 1
       end
     }
-    if fence_count == 0
+    if fence_count == 0 and sbd_enabled.empty?
       status[:warning_list] << {
-        :message => 'No fence devices configured in the cluster',
+        :message => 'No fencing configured in the cluster',
       }
     end
 
@@ -1557,6 +1574,30 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
         :message => 'Stonith is not enabled',
       }
     end
+    if not sbd_enabled.empty? and not sbd_disabled_node_list.empty?
+      status[:warning_list] << {
+        :message =>
+          "SBD is not enabled on node(s) #{sbd_disabled_node_list.join(', ')}",
+        :type => 'sbd_not_enabled_on_all_nodes',
+        :node_list => sbd_disabled_node_list
+      }
+    end
+    if not sbd_enabled.empty? and sbd_running.empty?
+      # if there is SBD running on at least one node, SBD has to be running
+      # on all online/standby nodes in cluster (it is impossible to have
+      # online node without running SBD, pacemaker will shutdown/not start
+      # in case like this)
+      status[:warning_list] << {
+        :message =>
+          'SBD is enabled but not running. Restart of cluster is required.',
+      }
+    end
+    if sbd_enabled.empty? and not sbd_running.empty?
+      status[:warning_list] << {
+        :message =>
+          'SBD is disabled but still running. Restart of cluster is required.',
+      }
+    end
   end
 
   if not_authorized_nodes.length > 0
@@ -1639,7 +1680,8 @@ def get_node_status(auth_user, cib_dom)
       :fence_levels => get_fence_levels(auth_user, cib_dom),
       :node_attr => node_attrs_to_v2(get_node_attributes(auth_user, cib_dom)),
       :nodes_utilization => get_nodes_utilization(cib_dom),
-      :known_nodes => []
+      :known_nodes => [],
+      :available_features => ['sbd', 'ticket_constraints']
   }
 
   nodes = get_nodes_status()
@@ -1816,7 +1858,7 @@ def status_v1_to_v2(status)
 end
 
 def allowed_for_local_cluster(auth_user, action)
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   return pcs_config.permissions_local.allows?(
     auth_user[:username], auth_user[:usergroups], action
   )
@@ -1848,3 +1890,118 @@ def get_default_overview_node_list(clustername)
   }
   return node_list
 end
+
+def is_service_enabled?(service)
+  if ISSYSTEMCTL
+    cmd = ['systemctl', 'is-enabled', "#{service}.service"]
+  else
+    cmd = ['chkconfig', service]
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  return (retcode == 0)
+end
+
+def is_service_running?(service)
+  if ISSYSTEMCTL
+    cmd = ['systemctl', 'status', "#{service}.service"]
+  else
+    cmd = ['service', service, 'status']
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  return (retcode == 0)
+end
+
+def is_service_installed?(service)
+  unless ISSYSTEMCTL
+    stdout, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), 'chkconfig')
+    if retcode != 0
+      return nil
+    end
+    stdout.each { |line|
+      if line.split(' ')[0] == service
+        return true
+      end
+      return false
+    }
+  end
+  stdout, _, retcode = run_cmd(
+    PCSAuth.getSuperuserAuth(), PCS, 'resource', 'list', 'systemd'
+  )
+  if retcode != 0
+    return nil
+  end
+  stdout.each { |line|
+    if line.strip() == "systemd:#{service}"
+      return true
+    end
+  }
+  return false
+end
+
+def enable_service(service)
+  if ISSYSTEMCTL
+    # fails when the service is not installed
+    cmd = ['systemctl', 'enable', "#{service}.service"]
+  else
+    # fails when the service is not installed
+    cmd = ['chkconfig', service, 'on']
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  return (retcode == 0)
+end
+
+def disable_service(service)
+  if ISSYSTEMCTL
+    # returns success even if the service is not installed
+    cmd = ['systemctl', 'disable', "#{service}.service"]
+  else
+    if not is_service_installed?(service)
+      return true
+    end
+    # fails when the service is not installed, so we need to check it beforehand
+    cmd = ['chkconfig', service, 'off']
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  return (retcode == 0)
+end
+
+def start_service(service)
+  _, _, retcode = run_cmd(
+    PCSAuth.getSuperuserAuth(), "service", service, "start"
+  )
+  return (retcode == 0)
+end
+
+def stop_service(service)
+  if not is_service_installed?(service)
+    return true
+  end
+  _, _, retcode = run_cmd(
+    PCSAuth.getSuperuserAuth(), "service", service, "stop"
+  )
+  return (retcode == 0)
+end
+
+def set_cluster_prop_force(auth_user, prop, val)
+  cmd = [PCS, 'property', 'set', "#{prop}=#{val}", '--force']
+  if pacemaker_running?
+    _, _, retcode = run_cmd(auth_user, *cmd)
+  else
+    cmd += ['-f', CIB_PATH]
+    _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  end
+  return (retcode == 0)
+end
+
+def get_parsed_local_sbd_config()
+  cmd = [PCS, 'stonith', 'sbd', 'local_config_in_json']
+  out, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+  if retcode != 0
+    return nil
+  end
+  begin
+    return JSON.parse(out.join(' '))
+  rescue JSON::ParserError
+    return nil
+  end
+end
