@@ -7,6 +7,8 @@ require 'webrick/https'
 require 'openssl'
 require 'logger'
 require 'thread'
+require 'fileutils'
+require 'cgi'
 
 require 'bootstrap.rb'
 require 'resource.rb'
@@ -44,7 +46,7 @@ rescue Errno::ENOENT
   File.open(COOKIE_FILE, 'w', 0700) {|f| f.write(secret)}
 end
 
-session_lifetime = ENV['SESSION_LIFETIME'].to_i()
+session_lifetime = ENV['PCSD_SESSION_LIFETIME'].to_i()
 session_lifetime = 60 * 60 unless session_lifetime > 0
 use SessionPoolLifetime,
   :expire_after => session_lifetime,
@@ -81,14 +83,16 @@ before do
     $session_storage_env = env
   end
 
-  if request.path != '/login' and not request.path == "/logout" and not request.path == '/remote/auth'
+  if request.path != '/login' and not request.path == "/logout" and not request.path == '/remote/auth' and not request.path == '/login-status'
     protected! 
   end
   $cluster_name = get_cluster_name()
 end
 
 configure do
-  DISABLE_GUI = (ENV['DISABLE_GUI'] and ENV['DISABLE_GUI'].downcase == 'true')
+  DISABLE_GUI = (
+    ENV['PCSD_DISABLE_GUI'] and ENV['PCSD_DISABLE_GUI'].downcase == 'true'
+  )
   PCS = get_pcs_path(File.expand_path(File.dirname(__FILE__)))
   logger = File.open("/var/log/pcsd/pcsd.log", "a+", 0600)
   STDOUT.reopen(logger)
@@ -365,7 +369,19 @@ if not DISABLE_GUI
 
   get '/logout' do
     session.destroy
-    redirect '/login'
+    if is_ajax?
+      halt [200, "OK"]
+    else
+      redirect '/login'
+    end
+  end
+
+  get '/login-status' do
+    if PCSAuth.isLoggedIn(session)
+      halt [200, session[:random]]
+    else
+      halt [401, '{"notauthorized":"true"}']
+    end
   end
 
   post '/login' do
@@ -386,8 +402,9 @@ if not DISABLE_GUI
       #      redirect plp
       #    else
       session.delete(:bad_login_name)
+      session[:random] = "#{Time.now.to_i}-#{rand(100)}"
       if is_ajax?
-        halt [200, "OK"]
+        halt [200, session[:random]]
       else
         redirect '/manage'
       end
@@ -403,7 +420,7 @@ if not DISABLE_GUI
   end
 
   post '/manage/existingcluster' do
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     node = params['node-name']
     code, result = send_request_with_token(
       PCSAuth.getSuperuserAuth(), node, 'status'
@@ -421,7 +438,7 @@ if not DISABLE_GUI
       nodes = status["corosync_offline"] + status["corosync_online"]
 
       if status["cluster_name"] == ''
-        return 400, "The node, '#{noname}', does not currently have a cluster
+        return 400, "The node, '#{node}', does not currently have a cluster
  configured.  You must create a cluster using this node before adding it to pcsd."
       end
 
@@ -446,7 +463,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
           return 400, "Unable to get authentication info from cluster '#{status['cluster_name']}'."
         end
 
-        sync_config = Cfgsync::PcsdTokens.from_file('')
+        sync_config = Cfgsync::PcsdTokens.from_file()
         pushed, _ = Cfgsync::save_sync_new_tokens(
           sync_config, new_tokens, get_corosync_nodes(), $cluster_name
         )
@@ -479,7 +496,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
 
     warning_messages = []
 
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     @manage = true
     @cluster_name = params[:clustername]
     @nodes = []
@@ -553,7 +570,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
         # we are waiting for the request to finish, so no locking is needed.
         # If we are in a different cluster we just try twice to update the
         # config, dealing with any updates in between.
-        pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+        pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
         pcs_config.clusters << Cluster.new(@cluster_name, @nodes)
         sync_config = Cfgsync::PcsdSettings.from_text(pcs_config.text())
         pushed, _ = Cfgsync::save_sync_new_version(
@@ -572,7 +589,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
   end
 
   post '/manage/removecluster' do
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     params.each { |k,v|
       if k.start_with?("clusterid-")
         pcs_config.remove_cluster(k.sub("clusterid-",""))
@@ -671,7 +688,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
 
     if not new_tokens.empty?
       cluster_nodes = get_corosync_nodes()
-      tokens_cfg = Cfgsync::PcsdTokens.from_file('')
+      tokens_cfg = Cfgsync::PcsdTokens.from_file()
       sync_successful, sync_responses = Cfgsync::save_sync_new_tokens(
         tokens_cfg, new_tokens, cluster_nodes, $cluster_name
       )
@@ -691,7 +708,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
 
   get '/permissions/?' do
     @manage = true
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     @clusters = pcs_config.clusters.sort { |a, b| a.name <=> b.name }
     erb :permissions, :layout => :main
   end
@@ -705,7 +722,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
     @user_types = []
     @users_permissions = []
 
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
 
     if not pcs_config.is_cluster_name_in_use(@cluster_name)
       @error = 'Cluster not found'
@@ -741,7 +758,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
   get '/managec/:cluster/main' do
     auth_user = PCSAuth.sessionToAuthUser(session)
     @cluster_name = params[:cluster]
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     @clusters = pcs_config.clusters
     @nodes = get_cluster_nodes(params[:cluster])
     if @nodes == []
@@ -946,6 +963,179 @@ already been added to pcsd.  You may not add two clusters with the same name int
     end
   end
 
+  get '/managec/:cluster/get_resource_agent_metadata' do
+    auth_user = PCSAuth.sessionToAuthUser(session)
+    cluster = params[:cluster]
+    resource_agent = params[:agent]
+    code, out = send_cluster_request_with_token(
+      auth_user,
+      cluster,
+      'get_resource_agent_metadata',
+      false,
+      {:resource_agent => resource_agent}
+    )
+    if code != 404
+      return [code, out]
+    end
+
+    code, out = send_cluster_request_with_token(
+      auth_user,
+      cluster,
+      'resource_metadata',
+      false,
+      {
+        :resourcename => resource_agent,
+        :new => true
+      }
+    )
+    if code != 200
+      return [400, 'Unable to get meta-data of specified resource agent.']
+    end
+    desc_regex = Regexp.new(
+      '<span class="reg[^>]*>(?<short>[^>]*)&nbsp;</span>[^<]*' +
+        '<span title="(?<long>[^"]*)"'
+    )
+    parameters_regex = Regexp.new(
+      '<input type="hidden" name="resource_type"[^>]*>(?<required>[\s\S]*)' +
+        '<div class="bold">Optional Arguments:</div>(?<optional>[\S\s]*)' +
+        '<tr class="stop">'
+    )
+    parameter_regex = Regexp.new(
+      '<tr title="(?<longdesc>[^"]*)"[^>]*>[\s]*<td class="reg">\s*' +
+        '(?<name>[^<\s]*)\s*</td>\s*<td>\s*' +
+        '<input placeholder="(?<shortdesc>[^"]*)"'
+    )
+
+    desc = desc_regex.match(out)
+    unless desc
+      return [400, 'Unable to get meta-data of specified resource agent.']
+    end
+    result = {
+      :name => resource_agent,
+      :shortdesc => html2plain(desc[:short]),
+      :longdesc => html2plain(desc[:long]),
+      :parameters => []
+    }
+
+    parameters = parameters_regex.match(out)
+    parameters[:required].scan(parameter_regex) { |match|
+      result[:parameters] << {
+        :name => html2plain(match[1]),
+        :longdesc => html2plain(match[0]),
+        :shortdesc => html2plain(match[2]),
+        :type => 'string',
+        :required => true
+      }
+    }
+    parameters[:optional].scan(parameter_regex) { |match|
+      result[:parameters] << {
+        :name => html2plain(match[1]),
+        :longdesc => html2plain(match[0]),
+        :shortdesc => html2plain(match[2]),
+        :type => 'string',
+        :required => false
+      }
+    }
+    return [200, JSON.generate(result)]
+  end
+
+  get '/managec/:cluster/get_fence_agent_metadata' do
+    auth_user = PCSAuth.sessionToAuthUser(session)
+    cluster = params[:cluster]
+    fence_agent = params[:agent]
+    code, out = send_cluster_request_with_token(
+      auth_user,
+      cluster,
+      'get_fence_agent_metadata',
+      false,
+      {:fence_agent => fence_agent}
+    )
+    if code != 404
+      return [code, out]
+    end
+
+    code, out = send_cluster_request_with_token(
+      auth_user,
+      cluster,
+      'fence_device_metadata',
+      false,
+      {
+        :resourcename => fence_agent.sub('stonith:', ''),
+        :new => true
+      }
+    )
+    if code != 200
+      return [400, 'Unable to get meta-data of specified fence agent.']
+    end
+    desc_regex = Regexp.new(
+      '<span class="reg[^>]*>(?<short>[^>]*)&nbsp;</span>[^<]*' +
+        '<span title="(?<long>[^"]*)"'
+    )
+    parameters_regex = Regexp.new(
+      '<input type="hidden" name="resource_type"[^>]*>(?<required>[\s\S]*)' +
+        '<div class="bold">Optional Arguments:</div>(?<optional>[\S\s]*)' +
+        '<div class="bold">Advanced Arguments:</div>(?<advanced>[\S\s]*)' +
+        '<tr class="stop">'
+    )
+    required_parameter_regex = Regexp.new(
+      '<tr title="(?<longdesc>[^"]*)[^>]*>[\s]*' +
+        '<td class="reg">\s*&nbsp;(?<name>[^<\s]*)\s*</td>\s*<td>\s*' +
+        '<input placeholder="(?<shortdesc>[^"]*)"'
+    )
+    other_parameter_regex = Regexp.new(
+      '<td class="reg">\s*&nbsp;(?<name>[^<\s]*)\s*</td>\s*<td>\s*' +
+        '<input placeholder="(?<shortdesc>[^"]*)"'
+    )
+
+    result = {
+      :name => fence_agent,
+      :shortdesc => '',
+      :longdesc => '',
+      :parameters => []
+    }
+
+    # pcsd in version 0.9.137 (and older) does not provide description for
+    # fence agents
+    desc = desc_regex.match(out)
+    if desc
+      result[:shortdesc] = html2plain(desc[:short])
+      result[:longdesc] = html2plain(desc[:long])
+    end
+
+    parameters = parameters_regex.match(out)
+    parameters[:required].scan(required_parameter_regex) { |match|
+      result[:parameters] << {
+        :name => html2plain(match[1]),
+        :longdesc => html2plain(match[0]),
+        :shortdesc => html2plain(match[2]),
+        :type => 'string',
+        :required => true,
+        :advanced => false
+      }
+    }
+    parameters[:optional].scan(other_parameter_regex) { |match|
+      result[:parameters] << {
+        :name => html2plain(match[0]),
+        :longdesc => '',
+        :shortdesc => html2plain(match[1]),
+        :type => 'string',
+        :required => false,
+        :advanced => false
+      }
+    }
+    parameters[:advanced].scan(other_parameter_regex) { |match|
+      result[:parameters] << {
+        :name => html2plain(match[0]),
+        :longdesc => '',
+        :shortdesc => html2plain(match[1]),
+        :type => 'string',
+        :required => false,
+        :advanced => true
+      }
+    }
+    return [200, JSON.generate(result)]
+  end
+
   post '/managec/:cluster/fix_auth_of_cluster' do
     clustername = params[:cluster]
     unless clustername
@@ -1097,6 +1287,10 @@ class Node
   end
 end
 
+def html2plain(text)
+  return CGI.unescapeHTML(text).gsub(/<br[^>]*>/, "\n")
+end
+
 helpers do
   def h(text)
     Rack::Utils.escape_html(text)
