@@ -234,20 +234,25 @@ function create_node(form) {
 // If update is set to true we update the resource instead of create it
 // if stonith is set to true we update/create a stonith agent
 function create_resource(form, update, stonith) {
-  dataString = $(form).serialize();
-  var resourceID = $(form).find("[name='name']").val(); 
-  url = get_cluster_remote_url() + $(form).attr("action");
+  var data = {};
+  $($(form).serializeArray()).each(function(index, obj) {
+    data[obj.name] = obj.value;
+  });
+  data["resource_type"] = data["resource_type"].replace("::", ":");
+  var url = get_cluster_remote_url() + $(form).attr("action");
   var name;
 
-  if (stonith)
+  if (stonith) {
     name = "fence device";
-  else
-    name = "resource"
+    data["resource_type"] = data["resource_type"].replace("stonith:", "");
+  } else {
+    name = "resource";
+  }
 
   ajax_wrapper({
     type: "POST",
     url: url,
-    data: dataString,
+    data: data,
     dataType: "json",
     success: function(returnValue) {
       $('input.apply_changes').show();
@@ -257,9 +262,9 @@ function create_resource(form, update, stonith) {
         Pcs.update();
         if (!update) {
           if (stonith)
-            $('#add_stonith').dialog('close');
+            $('#new_stonith_agent').dialog('close');
           else
-            $('#add_resource').dialog('close');
+            $('#new_resource_agent').dialog('close');
         } else {
           reload_current_resource();
         }
@@ -291,37 +296,15 @@ function disable_spaces(item) {
   });
 }
 
-function load_resource_form(item, ra, stonith) {
-  var data = { new: true, resourcename: ra};
-  var command;
-  if (!stonith)
-    command = "resource_metadata";
-  else
-    command = "fence_device_metadata";
-  
-  item.load(get_cluster_remote_url() + command, data);
-}
-
-function update_resource_form_groups(form, group_list) {
-  var select = $(form).find("select[name='resource_group']").first();
-  if (select.length < 1) {
+function load_resource_form(agent_name, stonith) {
+  stonith = typeof stonith !== 'undefined' ? stonith : false;
+  if (!agent_name) {
     return;
   }
-  var selected = select.val();
-  var selected_valid = false;
-  var select_new = select.clone();
-  select_new.empty();
-  select_new.append('<option value="">None</options>');
-  $.each(group_list, function(index, group) {
-    select_new.append('<option value="' + group + '">' + group + '</options>');
-    if (selected == group) {
-      selected_valid = true;
-    }
-  });
-  if (selected_valid) {
-    select_new.val(selected);
-  }
-  select.replaceWith(select_new);
+  var prop_name = "new_" + (stonith ? "fence" : "resource") + "_agent_metadata";
+  get_resource_agent_metadata(agent_name, function (data) {
+      Pcs.resourcesContainer.set(prop_name, Pcs.ResourceAgent.create(data));
+  }, stonith);
 }
 
 function verify_remove(remove_func, forceable, checklist_id, dialog_id, label, ok_text, title, remove_id) {
@@ -473,7 +456,7 @@ function setNodeStatus(node, running) {
     $('.node_name:contains("'+node+'")').css('color','red');
   }
 }
-  
+
 
 function fade_in_out(id) {
   $(id).fadeTo(1000, 0.01, function() {
@@ -1184,36 +1167,6 @@ function load_row(node_row, ac, cur_elem, containing_elem, also_set, initial_loa
   });
 }
 
-function load_agent_form(resource_id, stonith) {
-  var url;
-  var form;
-  if (stonith) {
-    form = $("#stonith_agent_form");
-    url = '/managec/' + Pcs.cluster_name + '/fence_device_form';
-  } else {
-    form = $("#resource_agent_form");
-    url = '/managec/' + Pcs.cluster_name + '/resource_form?version=2';
-  }
-
-  form.empty();
-
-  var resource_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
-  if (!resource_obj || !resource_obj.get('is_primitive'))
-    return;
-
-  var data = {resource: resource_id};
-
-  ajax_wrapper({
-    type: 'GET',
-    url: url,
-    data: data,
-    timeout: pcs_timeout,
-    success: function (data) {
-      Ember.run.next(function(){form.html(data);});
-    }
-  });
-}
-
 function show_loading_screen() {
   $("#loading_screen_progress_bar").progressbar({ value: 100});
   $("#loading_screen").dialog({
@@ -1517,18 +1470,35 @@ function add_meta_attr(parent_id) {
   Pcs.resourcesContainer.update_meta_attr(resource_id, attr, value);
 }
 
+
+function add_constraint_prepare_data(parent_id, constraint_type){
+  var value = function(sibling){
+    var form_value = $(parent_id + " " + sibling).val();
+    return form_value ? form_value.trim() : form_value;
+  };
+  switch(constraint_type){
+    case "ticket": return {
+      ticket: value("input[name='ticket']"),
+      role: value("select[name='role']"),
+      "loss-policy": value("select[name='loss-policy']"),
+    };
+  }
+  return {
+    rule: value("input[name='node_id']"),
+    score: value("input[name='score']"),
+    target_res_id: value("input[name='target_res_id']"),
+    order: value("select[name='order']"),
+    target_action: value("select[name='target_action']"),
+    res_action: value("select[name='res_action']"),
+    colocation_type: value("select[name='colocate']"),
+  };
+}
+
 function add_constraint(parent_id, c_type, force) {
-  var data = {};
+  var data = add_constraint_prepare_data(parent_id, c_type);
   data["disable_autocorrect"] = true;
   data["res_id"] = Pcs.resourcesContainer.cur_resource.get('id');
   data["node_id"] = $(parent_id + " input[name='node_id']").val();
-  data["rule"] = $(parent_id + " input[name='node_id']").val();
-  data["score"] = $(parent_id + " input[name='score']").val();
-  data["target_res_id"] = $(parent_id + " input[name='target_res_id']").val();
-  data["order"] = $(parent_id + " select[name='order']").val();
-  data["target_action"] = $(parent_id + " select[name='target_action']").val();
-  data["res_action"] = $(parent_id + " select[name='res_action']").val();
-  data["colocation_type"] = $(parent_id + " select[name='colocate']").val();
   data["c_type"] = c_type;
   if (force) {
     data["force"] = force;
@@ -1574,10 +1544,21 @@ function add_constraint(parent_id, c_type, force) {
   });
 }
 
+function add_constraint_set_get_options(parent_id, constraint_type){
+  switch(constraint_type){
+    case "ticket": return {
+      ticket: $(parent_id + " input[name='ticket']").val().trim(),
+      "loss-policy": $(parent_id + " select[name='loss-policy']").val().trim(),
+    };
+  }
+  return {};
+}
+
 function add_constraint_set(parent_id, c_type, force) {
   var data = {
     resources: [],
-    disable_autocorrect: true
+    disable_autocorrect: true,
+    options: {},
   };
   $(parent_id + " input[name='resource_ids[]']").each(function(index, element) {
     var resources = element.value.trim();
@@ -1585,6 +1566,7 @@ function add_constraint_set(parent_id, c_type, force) {
       data['resources'].push(resources.split(/\s+/));
     }
   });
+  data.options = add_constraint_set_get_options(parent_id, c_type);
   data["c_type"] = c_type;
   if (force) {
     data["force"] = force;
@@ -1660,6 +1642,11 @@ function remove_constraint(id) {
   });
 }
 
+function remove_constraint_action(remover_element){
+  remove_constraint($(remover_element).parent().attr('constraint_id'));
+  return false;
+}
+
 function remove_constraint_rule(id) {
   fade_in_out($("[rule_id='"+id+"']").parent());
   ajax_wrapper({
@@ -1805,7 +1792,7 @@ function remove_acl_item(id,item) {
 function update_cluster_settings() {
   $("#cluster_properties button").prop("disabled", true);
   var data = {
-    'hidden[hidden_input]': null // this is needed for backward compatibility 
+    'hidden[hidden_input]': null // this is needed for backward compatibility
   };
   $.each(Pcs.settingsController.get("properties"), function(_, prop) {
     data[prop.get("form_name")] = prop.get("cur_val");
@@ -1841,11 +1828,7 @@ function refresh_cluster_properties() {
       Pcs.settingsController.update(data);
     },
     error: function (xhr, status, error) {
-      alert(
-        "Unable to get cluster properties: "
-        + ajax_simple_error(xhr, status, error)
-      );
-      Pcs.settingsController.update({});
+      Pcs.settingsController.set("error", true);
     },
     complete: function() {
       hide_loading_screen();
@@ -2058,7 +2041,15 @@ function get_list_view_element_id(element) {
 }
 
 function auto_show_hide_constraints() {
-  var cont = ["location_constraints", "ordering_constraints", "ordering_set_constraints", "colocation_constraints", "meta_attributes"];
+  var cont = [
+    "location_constraints",
+    "ordering_constraints",
+    "ordering_set_constraints",
+    "colocation_constraints",
+    "ticket_constraints",
+    "ticket_set_constraints",
+    "meta_attributes",
+  ];
   $.each(cont, function(index, name) {
     var elem = $("#" + name)[0];
     var cur_resource = Pcs.resourcesContainer.get('cur_resource');
@@ -2072,6 +2063,40 @@ function auto_show_hide_constraints() {
   });
 }
 
+function get_resource_agent_metadata(agent, on_success, stonith) {
+  stonith = typeof stonith !== 'undefined' ? stonith : false;
+  var request = (stonith)
+    ? 'get_fence_agent_metadata'
+    : 'get_resource_agent_metadata';
+  ajax_wrapper({
+    url: get_cluster_remote_url() + request,
+    dataType: "json",
+    data: {agent: agent},
+    timeout: pcs_timeout,
+    success: on_success,
+    error: function (xhr, status, error) {
+      alert(
+        "Unable to get metadata for resource agent '" + agent + "' "
+        + ajax_simple_error(xhr, status, error)
+      );
+    }
+  })
+}
+
+function update_instance_attributes(resource_id) {
+  var res_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
+  if (!(res_obj && res_obj.get("is_primitive"))) {
+    return;
+  }
+  get_resource_agent_metadata(res_obj.get("resource_type"), function(data) {
+    var agent = Pcs.ResourceAgent.create(data);
+    res_obj.set("resource_agent", agent);
+    $.each(res_obj.get("instance_attr"), function(_, attr) {
+      agent.get_parameter(attr.name).set("value", attr.value);
+    });
+  }, res_obj.get("stonith"));
+}
+
 function tree_view_onclick(resource_id, auto) {
   auto = typeof auto !== 'undefined' ? auto : false;
   var resource_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
@@ -2081,18 +2106,21 @@ function tree_view_onclick(resource_id, auto) {
   }
   if (resource_obj.get('stonith')) {
     Pcs.resourcesContainer.set('cur_fence', resource_obj);
-    if (!auto) window.location.hash = "/fencedevices/" + resource_id;
+    if (!auto) {
+      window.location.hash = "/fencedevices/" + resource_id;
+      update_instance_attributes(resource_id);
+    }
   } else {
     Pcs.resourcesContainer.set('cur_resource', resource_obj);
-    if (!auto) window.location.hash = "/resources/" + resource_id;
+
+    if (!auto) {
+      window.location.hash = "/resources/" + resource_id;
+      update_instance_attributes(resource_id);
+    }
     auto_show_hide_constraints();
   }
 
   tree_view_select(resource_id);
-
-  Ember.run.next(Pcs, function() {
-    load_agent_form(resource_id, resource_obj.get('stonith'));
-  });
 }
 
 function tree_view_select(element_id) {
@@ -2228,10 +2256,10 @@ function resource_change_group(resource_id, group_id) {
     resource_id: resource_id,
     group_id: group_id
   };
-  
+
   if (resource_obj.get('parent')) {
     if (resource_obj.get('parent').get('id') == group_id) {
-      return;  
+      return;
     }
     if (resource_obj.get('parent').get('class_type') == 'group') {
       data['old_group_id'] = resource_obj.get('parent').get('id');
@@ -2723,7 +2751,7 @@ Ember.Handlebars.helper('selector-helper', function (content, value, place_holde
   var out = "";
   var line;
   if (place_holder) {
-    out += '<option value="">' + place_holder + '</option>'; 
+    out += '<option value="">' + place_holder + '</option>';
   }
   $.each(content, function(_, opt){
     line = '<option value="' + opt["value"] + '"';
@@ -2735,3 +2763,189 @@ Ember.Handlebars.helper('selector-helper', function (content, value, place_holde
   });
   return new Handlebars.SafeString(out);
 });
+
+Ember.Handlebars.helper('bool-to-icon', function(value, options) {
+  var out = '<span class="sprites inverted ';
+  if (typeof(value) == 'undefined' || value == null) {
+    out += "questionmarkdark";
+  } else if (value) {
+    out += "checkdark"
+  } else {
+    out += "Xdark"
+  }
+  return new Handlebars.SafeString(out + '">&nbsp;</span>');
+});
+
+function nl2br(text) {
+  return text.replace(/(?:\r\n|\r|\n)/g, '<br />');
+}
+
+function enable_sbd(dialog) {
+  ajax_wrapper({
+    type: 'POST',
+    url: get_cluster_remote_url() + "remote_enable_sbd",
+    data: dialog.find("#enable_sbd_form").serialize(),
+    timeout: pcs_timeout,
+    success: function() {
+      dialog.parent().find("#enable_sbd_btn").button(
+        "option", "disabled", false
+      );
+      dialog.dialog("close");
+      alert(
+        'SBD enabled! You have to restart cluster in order to apply changes.'
+      );
+      Pcs.update();
+    },
+    error: function (xhr, status, error) {
+      dialog.parent().find("#enable_sbd_btn").button(
+        "option", "disabled", false
+      );
+      xhr.responseText = xhr.responseText.replace(
+        "--skip-offline", "option 'ignore offline nodes'"
+      );
+      alert(
+        ajax_simple_error(xhr, status, error)
+      );
+    }
+  });
+}
+
+function enable_sbd_dialog(node_list) {
+  var buttonsOpts = [
+    {
+      text: "Enable SBD",
+      id: "enable_sbd_btn",
+      click: function() {
+        var dialog = $(this);
+        dialog.parent().find("#enable_sbd_btn").button(
+          "option", "disabled", true
+        );
+        enable_sbd(dialog);
+      }
+    },
+    {
+      text:"Cancel",
+      click: function () {
+        $(this).dialog("close");
+      }
+    }
+  ];
+
+  var dialog_obj = $("#enable_sbd_dialog").dialog({title: 'Enable SBD',
+    modal: true, resizable: false,
+    width: 'auto',
+    buttons: buttonsOpts
+  });
+
+  dialog_obj.keypress(function(e) {
+    if (
+      e.keyCode == $.ui.keyCode.ENTER &&
+      !dialog_obj.parent().find("#enable_sbd_btn").button("option", "disabled")
+    ) {
+      dialog_obj.parent().find("#enable_sbd_btn").trigger("click");
+      return false;
+    }
+  });
+  dialog_obj.find('#watchdog_table').empty();
+  $.each(node_list, function(_, node) {
+    dialog_obj.find("#watchdog_table").append(
+      '<tr>' +
+        '<td>' +
+          node + ':' +
+        '</td>' +
+        '<td>' +
+          '<input ' +
+            'type="text" ' +
+            'placeholder="/dev/watchdog" ' +
+            'name="watchdog[' + node + ']" ' +
+          '/>' +
+        '</td>' +
+      '</tr>'
+    )
+  });
+}
+
+function disable_sbd(dialog) {
+  ajax_wrapper({
+    type: 'POST',
+    url: get_cluster_remote_url() + "remote_disable_sbd",
+    data: dialog.find("#disable_sbd_form").serialize(),
+    timeout: pcs_timeout,
+    success: function() {
+      dialog.parent().find("#disable_sbd_btn").button(
+        "option", "disabled", false
+      );
+      dialog.dialog("close");
+      alert(
+        'SBD disabled! You have to restart cluster in order to apply changes.'
+      );
+      Pcs.update();
+    },
+    error: function (xhr, status, error) {
+      dialog.parent().find("#disable_sbd_btn").button(
+        "option", "disabled", false
+      );
+      xhr.responseText = xhr.responseText.replace(
+        "--skip-offline", "option 'ignore offline nodes'"
+      );
+      alert(ajax_simple_error(xhr, status, error));
+    }
+  });
+}
+
+function disable_sbd_dialog() {
+  var buttonsOpts = [
+    {
+      text: "Disable SBD",
+      id: "disable_sbd_btn",
+      click: function() {
+        var dialog = $(this);
+        dialog.parent().find("#disable_sbd_btn").button(
+          "option", "disabled", true
+        );
+        disable_sbd(dialog);
+      }
+    },
+    {
+      text:"Cancel",
+      click: function () {
+        $(this).dialog("close");
+      }
+    }
+  ];
+
+  $("#disable_sbd_dialog").dialog({
+    title: 'Disable SBD',
+    modal: true, resizable: false,
+    width: 'auto',
+    buttons: buttonsOpts
+  });
+}
+
+function sbd_status_dialog() {
+  var buttonsOpts = [
+    {
+      text: "Enable SBD",
+      click: function() {
+        enable_sbd_dialog(Pcs.nodesController.get_node_name_list());
+      }
+    },
+    {
+      text: "Disable SBD",
+      click: disable_sbd_dialog
+    },
+    {
+      text:"Close",
+      click: function () {
+        $(this).dialog("close");
+      }
+    }
+  ];
+
+  $("#sbd_status_dialog").dialog({
+    title: 'SBD',
+    modal: true, resizable: false,
+    width: 'auto',
+    buttons: buttonsOpts
+  });
+}
