@@ -4,6 +4,8 @@ require 'open4'
 require 'set'
 require 'timeout'
 require 'rexml/document'
+require 'base64'
+require 'tempfile'
 
 require 'pcs.rb'
 require 'resource.rb'
@@ -62,18 +64,33 @@ def remote(params, request, auth_user)
       :get_tokens => method(:get_tokens),
       :get_cluster_tokens => method(:get_cluster_tokens),
       :save_tokens => method(:save_tokens),
-      :get_cluster_properties_definition => method(:get_cluster_properties_definition)
+      :get_cluster_properties_definition => method(:get_cluster_properties_definition),
+      :check_sbd => method(:check_sbd),
+      :set_sbd_config => method(:set_sbd_config),
+      :get_sbd_config => method(:get_sbd_config),
+      :sbd_disable => method(:sbd_disable),
+      :sbd_enable => method(:sbd_enable),
+      :remove_stonith_watchdog_timeout=> method(:remove_stonith_watchdog_timeout),
+      :set_stonith_watchdog_timeout_to_zero => method(:set_stonith_watchdog_timeout_to_zero),
+      :remote_enable_sbd => method(:remote_enable_sbd),
+      :remote_disable_sbd => method(:remote_disable_sbd),
+      :qdevice_net_get_ca_certificate => method(:qdevice_net_get_ca_certificate),
+      :qdevice_net_sign_node_certificate => method(:qdevice_net_sign_node_certificate),
+      :qdevice_net_client_init_certificate_storage => method(:qdevice_net_client_init_certificate_storage),
+      :qdevice_net_client_import_certificate => method(:qdevice_net_client_import_certificate),
+      :qdevice_net_client_destroy => method(:qdevice_net_client_destroy),
+      :qdevice_client_enable => method(:qdevice_client_enable),
+      :qdevice_client_disable => method(:qdevice_client_disable),
+      :qdevice_client_start => method(:qdevice_client_start),
+      :qdevice_client_stop => method(:qdevice_client_stop),
   }
   remote_cmd_with_pacemaker = {
+      :pacemaker_node_status => method(:remote_pacemaker_node_status),
       :resource_start => method(:resource_start),
       :resource_stop => method(:resource_stop),
       :resource_cleanup => method(:resource_cleanup),
-      :resource_form => method(:resource_form),
-      :fence_device_form => method(:fence_device_form),
       :update_resource => method(:update_resource),
       :update_fence_device => method(:update_fence_device),
-      :resource_metadata => method(:resource_metadata),
-      :fence_device_metadata => method(:fence_device_metadata),
       :get_avail_resource_agents => method(:get_avail_resource_agents),
       :get_avail_fence_agents => method(:get_avail_fence_agents),
       :remove_resource => method(:remove_resource),
@@ -97,23 +114,28 @@ def remote(params, request, auth_user)
       :resource_unclone => method(:resource_unclone),
       :resource_ungroup => method(:resource_ungroup),
       :set_resource_utilization => method(:set_resource_utilization),
-      :set_node_utilization => method(:set_node_utilization)
+      :set_node_utilization => method(:set_node_utilization),
+      :get_resource_agent_metadata => method(:get_resource_agent_metadata),
+      :get_fence_agent_metadata => method(:get_fence_agent_metadata)
   }
 
   command = params[:command].to_sym
-
-  if remote_cmd_without_pacemaker.include? command
-    return remote_cmd_without_pacemaker[command].call(
-      params, request, auth_user
-    )
-  elsif remote_cmd_with_pacemaker.include? command
-    if pacemaker_running?
-      return remote_cmd_with_pacemaker[command].call(params, request, auth_user)
+  begin
+    if remote_cmd_without_pacemaker.include? command
+      return remote_cmd_without_pacemaker[command].call(
+        params, request, auth_user
+      )
+    elsif remote_cmd_with_pacemaker.include? command
+      if pacemaker_running?
+        return remote_cmd_with_pacemaker[command].call(params, request, auth_user)
+      else
+        return [200,'{"pacemaker_not_running":true}']
+      end
     else
-      return [200,'{"pacemaker_not_running":true}']
+      return [404, "Unknown Request"]
     end
-  else
-    return [404, "Unknown Request"]
+  rescue NotImplementedException => e
+    return [501, "#{e}"]
   end
 end
 
@@ -133,7 +155,7 @@ def cluster_status_gui(auth_user, cluster_name, dont_update_config=false)
   new_cluster_nodes.uniq!
 
   if new_cluster_nodes.length > 0
-    config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     if !(dont_update_config or config.cluster_nodes_equal?(cluster_name, new_cluster_nodes))
       old_cluster_nodes = config.get_nodes(cluster_name)
       $logger.info("Updating node list for: #{cluster_name} #{old_cluster_nodes}->#{new_cluster_nodes}")
@@ -189,9 +211,13 @@ def cluster_start(params, request, auth_user)
       return 403, 'Permission denied'
     end
     $logger.info "Starting Daemons"
-    output =  `#{PCS} cluster start`
+    output, stderr, retval = run_cmd(auth_user, PCS, 'cluster', 'start')
     $logger.debug output
-    return output
+    if retval != 0
+      return [400, (output + stderr).join]
+    else
+      return output
+    end
   end
 end
 
@@ -310,7 +336,7 @@ def node_standby(params, request, auth_user)
       return 403, 'Permission denied'
     end
     $logger.info "Standby Node"
-    stdout, stderr, retval = run_cmd(auth_user, PCS, "cluster", "standby")
+    stdout, stderr, retval = run_cmd(auth_user, PCS, "node", "standby")
     return stdout
   end
 end
@@ -326,7 +352,7 @@ def node_unstandby(params, request, auth_user)
       return 403, 'Permission denied'
     end
     $logger.info "Unstandby Node"
-    stdout, stderr, retval = run_cmd(auth_user, PCS, "cluster", "unstandby")
+    stdout, stderr, retval = run_cmd(auth_user, PCS, "node", "unstandby")
     return stdout
   end
 end
@@ -624,7 +650,7 @@ def get_permissions_remote(params, request, auth_user)
     return 403, 'Permission denied'
   end
 
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   data = {
     'user_types' => Permissions::get_user_types(),
     'permission_types' => Permissions::get_permission_types(),
@@ -690,7 +716,7 @@ def set_permissions_remote(params, request, auth_user)
   perm_set = Permissions::PermissionsSet.new(perm_list)
 
   full_users_old = Set.new
-  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   pcs_config.permissions_local.entity_permissions_list.each{ |entity_perm|
     if entity_perm.allow_list.include?(Permissions::FULL)
       full_users_old << [entity_perm.type, entity_perm.name]
@@ -715,7 +741,7 @@ def set_permissions_remote(params, request, auth_user)
   end
 
   2.times {
-    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     pcs_config.permissions_local = perm_set
     sync_config = Cfgsync::PcsdSettings.from_text(pcs_config.text())
     pushed, _ = Cfgsync::save_sync_new_version(
@@ -763,7 +789,9 @@ def remote_add_node(params, request, auth_user, all=false)
     if params[:new_ring1addr] != nil
       node += ',' + params[:new_ring1addr]
     end
-    retval, output = add_node(auth_user, node, all, auto_start)
+    retval, output = add_node(
+      auth_user, node, all, auto_start, params[:watchdog]
+    )
   end
 
   if retval == 0
@@ -807,11 +835,15 @@ def remote_remove_nodes(params, request, auth_user)
     retval, output = remove_node(auth_user, node, true)
     out = out + output.join("\n")
   }
-  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   if config.get_nodes($cluster_name) == nil or config.get_nodes($cluster_name).length == 0
     return [200,"No More Nodes"]
   end
-  return out
+  if retval != 0
+    return [400, out]
+  else
+    return [200, out]
+  end
 end
 
 def remote_remove_node(params, request, auth_user)
@@ -904,6 +936,18 @@ def create_cluster(params, request, auth_user)
   end
 end
 
+def remote_pacemaker_node_status(params, request, auth_user)
+  if not allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  output, stderr, retval = run_cmd(auth_user, PCS, 'node', 'pacemaker-status')
+  if retval != 0
+    return [400, stderr]
+  else
+    return output
+  end
+end
+
 def node_status(params, request, auth_user)
   if params[:node] and params[:node] != '' and params[:node] !=
     $cur_node_name and !params[:redirected]
@@ -943,7 +987,7 @@ def node_status(params, request, auth_user)
 
   if not_authorized_nodes.length > 0
     node.warning_list << {
-      :message => 'Not authorized against node(s) ' + 
+      :message => 'Not authorized against node(s) ' +
         not_authorized_nodes.join(', '),
       :type => 'nodes_not_authorized',
       :node_list => not_authorized_nodes,
@@ -1058,7 +1102,7 @@ def status_all(params, request, auth_user, nodes=[], dont_update_config=false)
 
   node_list.uniq!
   if node_list.length > 0
-    config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+    config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
     old_node_list = config.get_nodes(params[:cluster])
     if !(dont_update_config or config.cluster_nodes_equal?(params[:cluster], node_list))
       $logger.info("Updating node list for: #{params[:cluster]} #{old_node_list}->#{node_list}")
@@ -1080,7 +1124,7 @@ def clusters_overview(params, request, auth_user)
   cluster_map = {}
   forbidden_clusters = {}
   threads = []
-  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   config.clusters.each { |cluster|
     threads << Thread.new {
       cluster_map[cluster.name] = {
@@ -1212,7 +1256,7 @@ def clusters_overview(params, request, auth_user)
 
   # update clusters in PCSConfig
   not_current_data = false
-  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file('{}').text())
+  config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   cluster_map.each { |cluster, values|
     next if forbidden_clusters[cluster]
     nodes = []
@@ -1344,56 +1388,6 @@ def resource_start(params, request, auth_user)
   end
 end
 
-def resource_form(params, request, auth_user)
-  if not allowed_for_local_cluster(auth_user, Permissions::READ)
-    return 403, 'Permission denied'
-  end
-
-  cib_dom = get_cib_dom(auth_user)
-  @cur_resource = get_resource_by_id(params[:resource], cib_dom)
-  @groups = get_resource_groups(cib_dom)
-  @version = params[:version]
-
-  if @cur_resource.instance_of?(ClusterEntity::Primitive) and !@cur_resource.stonith
-    @cur_resource_group = @cur_resource.get_group
-    @cur_resource_clone = @cur_resource.get_clone
-    @cur_resource_ms = @cur_resource.get_master
-    @resource = ResourceAgent.new(@cur_resource.agentname)
-    if @cur_resource.provider == 'heartbeat'
-      @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, HEARTBEAT_AGENTS_DIR + @cur_resource.type)
-    elsif @cur_resource.provider == 'pacemaker'
-      @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, PACEMAKER_AGENTS_DIR + @cur_resource.type)
-    elsif @cur_resource._class == 'nagios'
-      @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, NAGIOS_METADATA_DIR + @cur_resource.type + '.xml')
-    end
-    @existing_resource = true
-    if @resource
-      erb :resourceagentform
-    else
-      "Can't find resource"
-    end
-  else
-    "Resource #{params[:resource]} doesn't exist"
-  end
-end
-
-def fence_device_form(params, request, auth_user)
-  if not allowed_for_local_cluster(auth_user, Permissions::READ)
-    return 403, 'Permission denied'
-  end
-
-  @cur_resource = get_resource_by_id(params[:resource], get_cib_dom(auth_user))
-
-  if @cur_resource.instance_of?(ClusterEntity::Primitive) and @cur_resource.stonith
-    @resource_agents = getFenceAgents(auth_user, @cur_resource.agentname)
-    @existing_resource = true
-    @fenceagent = @resource_agents[@cur_resource.type]
-    erb :fenceagentform
-  else
-    "Can't find fence device"
-  end
-end
-
 # Creates resource if params[:resource_id] is not set
 def update_resource (params, request, auth_user)
   if not allowed_for_local_cluster(auth_user, Permissions::WRITE)
@@ -1521,66 +1515,82 @@ def get_avail_fence_agents(params, request, auth_user)
   if not allowed_for_local_cluster(auth_user, Permissions::READ)
     return 403, 'Permission denied'
   end
-  agents = getFenceAgents(auth_user)
+  agents = getFenceAgents()
   return JSON.generate(agents)
 end
 
-def resource_metadata(params, request, auth_user)
-  if not allowed_for_local_cluster(auth_user, Permissions::READ)
-    return 403, 'Permission denied'
-  end
-  return 200 if not params[:resourcename] or params[:resourcename] == ""
-  resource_name = params[:resourcename][params[:resourcename].rindex(':')+1..-1]
-  class_provider = params[:resourcename][0,params[:resourcename].rindex(':')]
-
-  @resource = ResourceAgent.new(params[:resourcename])
-  if class_provider == "ocf:heartbeat"
-    @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, HEARTBEAT_AGENTS_DIR + resource_name)
-  elsif class_provider == "ocf:pacemaker"
-    @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, PACEMAKER_AGENTS_DIR + resource_name)
-  elsif class_provider == 'nagios'
-    @resource.required_options, @resource.optional_options, @resource.info = getResourceMetadata(auth_user, NAGIOS_METADATA_DIR + resource_name + '.xml')
-  end
-  @new_resource = params[:new]
-  @resources, @groups = getResourcesGroups(auth_user)
-
-  erb :resourceagentform
-end
-
-def fence_device_metadata(params, request, auth_user)
-  if not allowed_for_local_cluster(auth_user, Permissions::READ)
-    return 403, 'Permission denied'
-  end
-  return 200 if not params[:resourcename] or params[:resourcename] == ""
-  @fenceagent = FenceAgent.new(params[:resourcename])
-  @fenceagent.required_options, @fenceagent.optional_options, @fenceagent.advanced_options, @fenceagent.info = getFenceAgentMetadata(auth_user, params[:resourcename])
-  @new_fenceagent = params[:new]
-  
-  erb :fenceagentform
-end
-
 def remove_resource(params, request, auth_user)
   if not allowed_for_local_cluster(auth_user, Permissions::WRITE)
     return 403, 'Permission denied'
   end
   force = params['force']
+  user = PCSAuth.getSuperuserAuth()
   no_error_if_not_exists = params.include?('no_error_if_not_exists')
-  errors = ""
-  params.each { |k,v|
-    if k.index("resid-") == 0
-      resid = k.gsub('resid-', '')
-      command = [PCS, 'resource', 'delete', resid]
-      command << '--force' if force
-      out, errout, retval = run_cmd(auth_user, *command)
+  resource_list = []
+  errors = ''
+  resource_to_remove = []
+  params.each { |param,_|
+    if param.start_with?('resid-')
+      resource_list << param.split('resid-', 2)[1]
+    end
+  }
+  tmp_file = nil
+  if force
+    resource_to_remove = resource_list
+  else
+    begin
+      tmp_file = Tempfile.new('temp_cib')
+      _, err, retval = run_cmd(user, PCS, 'cluster', 'cib', tmp_file.path)
       if retval != 0
-        unless out.index(" does not exist.") != -1 and no_error_if_not_exists  
-          errors += errout.join(' ').strip + "\n"
+        return [400, 'Unable to stop resource(s).']
+      end
+      cmd = [PCS, '-f', tmp_file.path, 'resource', 'disable']
+      resource_list.each { |resource|
+        _, err, retval = run_cmd(user, *cmd, resource)
+        if retval != 0
+          unless (
+            err.join('').index('unable to find a resource') != -1 and
+            no_error_if_not_exists
+          )
+            errors += "Unable to stop resource '#{resource}': #{err.join('')}"
+          end
+        else
+          resource_to_remove << resource
         end
+      }
+      _, _, retval = run_cmd(
+        user, PCS, 'cluster', 'cib-push', tmp_file.path, '--config', '--wait'
+      )
+      if retval != 0
+        return [400, 'Unable to stop resource(s).']
+      end
+      errors.strip!
+      unless errors.empty?
+        $logger.info("Stopping resource(s) errors:\n#{errors}")
+        return [400, errors]
+      end
+    rescue IOError
+      return [400, 'Unable to stop resource(s).']
+    ensure
+      if tmp_file
+        tmp_file.close!
+      end
+    end
+  end
+  resource_to_remove.each { |resource|
+    cmd = [PCS, 'resource', 'delete', resource]
+    if force
+      cmd << '--force'
+    end
+    out, err, retval = run_cmd(auth_user, *cmd)
+    if retval != 0
+      unless out.index(' does not exist.') != -1 and no_error_if_not_exists
+        errors += err.join(' ').strip + "\n"
       end
     end
   }
   errors.strip!
-  if errors == ""
+  if errors.empty?
     return 200
   else
     $logger.info("Remove resource errors:\n"+errors)
@@ -1763,6 +1773,12 @@ def add_constraint_remote(params, request, auth_user)
       auth_user,
       resA, resB, score, params["force"], !params['disable_autocorrect']
     )
+  when "ticket"
+    retval, error = add_ticket_constraint(
+      auth_user,
+      params["ticket"], params["res_id"], params["role"], params["loss-policy"],
+      params["force"], !params['disable_autocorrect']
+    )
   else
     return [400, "Unknown constraint type: #{params['c_type']}"]
   end
@@ -1805,6 +1821,18 @@ def add_constraint_set_remote(params, request, auth_user)
       auth_user,
       params["resources"].values, params["force"], !params['disable_autocorrect']
     )
+  when "ticket"
+    unless params["options"]["ticket"]
+      return [400, "Error adding constraint ticket: option ticket missing"]
+    end
+    retval, error = add_ticket_set_constraint(
+      auth_user,
+      params["options"]["ticket"],
+      (params["options"]["loss-policy"] or ""),
+      params["resources"].values,
+      params["force"],
+      !params['disable_autocorrect']
+    )
   else
     return [400, "Unknown constraint type: #{params["c_type"]}"]
   end
@@ -1993,7 +2021,7 @@ def save_tokens(params, request, auth_user)
     end
   }
 
-  tokens_cfg = Cfgsync::PcsdTokens.from_file('')
+  tokens_cfg = Cfgsync::PcsdTokens.from_file()
   sync_successful, sync_responses = Cfgsync::save_sync_new_tokens(
     tokens_cfg, new_tokens, get_corosync_nodes(), $cluster_name
   )
@@ -2066,7 +2094,7 @@ def resource_ungroup(params, request, auth_user)
   unless params[:group_id]
     return [400, 'group_id has to be specified.']
   end
-  
+
   _, stderr, retval = run_cmd(
     auth_user, PCS, 'resource', 'ungroup', params[:group_id]
   )
@@ -2086,7 +2114,7 @@ def resource_clone(params, request, auth_user)
   unless params[:resource_id]
     return [400, 'resource_id has to be specified.']
   end
-  
+
   _, stderr, retval = run_cmd(
     auth_user, PCS, 'resource', 'clone', params[:resource_id]
   )
@@ -2180,3 +2208,384 @@ def get_cluster_properties_definition(params, request, auth_user)
   end
   return [400, '{}']
 end
+
+def get_resource_agent_metadata(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  agent = params[:resource_agent]
+  unless agent
+    return [400, 'Parameter "resource_agent" required.']
+  end
+  stdout, stderr, retval = run_cmd(
+    auth_user, PCS, 'resource', 'get_resource_agent_info', agent
+  )
+  if retval != 0
+    if stderr.join('').include?('is not supported')
+      return [200, JSON.generate({
+        :name => agent,
+        :longdesc => '',
+        :shortdesc => '',
+        :parameters => []
+      })]
+    else
+      return [400, stderr.join("\n")]
+    end
+  end
+  return [200, stdout.join("\n")]
+end
+
+def get_fence_agent_metadata(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  agent = params[:fence_agent]
+  unless agent
+    return [400, 'Parameter "fence_agent" required.']
+  end
+  stdout, stderr, retval = run_cmd(
+    auth_user, PCS, 'stonith', 'get_fence_agent_info', agent
+  )
+  if retval != 0
+    return [400, stderr.join("\n")]
+  end
+  return [200, stdout.join("\n")]
+end
+
+def check_sbd(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  out = {
+    :sbd => {
+      :installed => is_service_installed?('sbd'),
+      :enabled => is_service_enabled?('sbd'),
+      :running => is_service_running?('sbd')
+    }
+  }
+  watchdog = param[:watchdog]
+  if watchdog
+    out[:watchdog] = {
+      :path => watchdog,
+      :exist => File.exist?(watchdog)
+    }
+  end
+  return [200, JSON.generate(out)]
+end
+
+def set_sbd_config(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  config = param[:config]
+  unless config
+    return [400, 'Parameter "config" required']
+  end
+
+  file = nil
+  begin
+    file = File.open(SBD_CONFIG, 'w')
+    file.flock(File::LOCK_EX)
+    file.write(config)
+  rescue => e
+    msg = "Unable to save SBD configuration: #{e}"
+    $logger.error(msg)
+    return [400, msg]
+  ensure
+    if file
+      file.flock(File::LOCK_UN)
+      file.close()
+    end
+  end
+  msg = 'SBD configuration saved.'
+  $logger.info(msg)
+  return [200, msg]
+end
+
+def get_sbd_config(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  out = []
+  file = nil
+  begin
+    file = File.open(SBD_CONFIG, 'r')
+    file.flock(File::LOCK_SH)
+    out = file.readlines()
+  rescue => e
+    msg = "Unable to get SBD configuration: #{e}"
+    $logger.error(msg)
+    return [400, msg]
+  ensure
+    if file
+      file.flock(File::LOCK_UN)
+      file.close()
+    end
+  end
+  return [200, out.join('')]
+end
+
+def sbd_disable(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if disable_service('sbd')
+    msg = 'SBD disabled'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Disabling SBD failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
+
+def sbd_enable(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if enable_service('sbd')
+    msg = 'SBD enabled'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Enabling SBD failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
+
+def remove_stonith_watchdog_timeout(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if set_cluster_prop_force(auth_user, 'stonith-watchdog-timeout', '')
+    $logger.info('Cluster property "stonith-watchdog-timeout" removed')
+    return [200, 'OK']
+  else
+    $logger.info('Failed to remove cluster property "stonith-watchdog-timeout"')
+    return [400, 'ERROR']
+  end
+end
+
+def set_stonith_watchdog_timeout_to_zero(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if set_cluster_prop_force(auth_user, 'stonith-watchdog-timeout', '0')
+    $logger.info('Cluster property "stonith-watchdog-timeout" set to "0"')
+    return [200, 'OK']
+  else
+    $logger.info(
+      'Failed to set cluster property "stonith-watchdog-timeout"to 0'
+    )
+    return [400, 'ERROR']
+  end
+end
+
+def remote_enable_sbd(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+
+  arg_list = []
+
+  if ['true', '1', 'on'].include?(params[:ignore_offline_nodes])
+    arg_list << '--skip-offline'
+  end
+
+  params[:watchdog].each do |node, watchdog|
+    unless watchdog.strip.empty?
+      arg_list << "--watchdog=#{watchdog.strip}@#{node}"
+    end
+  end
+
+  params[:config].each do |option, value|
+    unless value.empty?
+      arg_list << "#{option}=#{value}"
+    end
+  end
+
+  _, stderr, retcode = run_cmd(
+    auth_user, PCS, 'stonith', 'sbd', 'enable', *arg_list
+  )
+
+  if retcode != 0
+    return [400, "Unable to enable sbd in cluster:\n#{stderr.join('')}"]
+  end
+
+  return [200, 'Sbd has been enabled.']
+end
+
+def remote_disable_sbd(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+
+  arg_list = []
+
+  if ['true', '1', 'on'].include?(params[:ignore_offline_nodes])
+    arg_list << '--skip-offline'
+  end
+
+  _, stderr, retcode = run_cmd(
+    auth_user, PCS, 'stonith', 'sbd', 'disable', *arg_list
+  )
+
+  if retcode != 0
+    return [400, "Unable to disable sbd in cluster:\n#{stderr.join('')}"]
+  end
+
+  return [200, 'Sbd has been disabled.']
+end
+
+def qdevice_net_get_ca_certificate(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  begin
+    return [
+      200,
+      Base64.encode64(File.read(COROSYNC_QDEVICE_NET_SERVER_CA_FILE))
+    ]
+  rescue => e
+    return [400, "Unable to read certificate: #{e}"]
+  end
+end
+
+def qdevice_net_sign_node_certificate(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  stdout, stderr, retval = run_cmd_options(
+    auth_user,
+    {'stdin' => params[:certificate_request]},
+    PCS, 'qdevice', 'sign-net-cert-request', '--name', params[:cluster_name]
+  )
+  if retval != 0
+    return [400, stderr.join('')]
+  end
+  return [200, stdout.join('')]
+end
+
+def qdevice_net_client_init_certificate_storage(params, request, auth_user)
+  # Last step of adding qdevice into a cluster is distribution of corosync.conf
+  # file with qdevice settings. This requires FULL permissions currently.
+  # If that gets relaxed, we can require lower permissions in here as well.
+  unless allowed_for_local_cluster(auth_user, Permissions::FULL)
+    return 403, 'Permission denied'
+  end
+  stdout, stderr, retval = run_cmd_options(
+    auth_user,
+    {'stdin' => params[:ca_certificate]},
+    PCS, 'qdevice', 'net-client', 'setup'
+  )
+  if retval != 0
+    return [400, stderr.join('')]
+  end
+  return [200, stdout.join('')]
+end
+
+def qdevice_net_client_import_certificate(params, request, auth_user)
+  # Last step of adding qdevice into a cluster is distribution of corosync.conf
+  # file with qdevice settings. This requires FULL permissions currently.
+  # If that gets relaxed, we can require lower permissions in here as well.
+  unless allowed_for_local_cluster(auth_user, Permissions::FULL)
+    return 403, 'Permission denied'
+  end
+  stdout, stderr, retval = run_cmd_options(
+    auth_user,
+    {'stdin' => params[:certificate]},
+    PCS, 'qdevice', 'net-client', 'import-certificate'
+  )
+  if retval != 0
+    return [400, stderr.join('')]
+  end
+  return [200, stdout.join('')]
+end
+
+def qdevice_net_client_destroy(param, request, auth_user)
+  # When removing a qdevice from a cluster, an updated corosync.conf file
+  # with removed qdevice settings is distributed. This requires FULL permissions
+  # currently. If that gets relaxed, we can require lower permissions in here
+  # as well.
+  unless allowed_for_local_cluster(auth_user, Permissions::FULL)
+    return 403, 'Permission denied'
+  end
+  stdout, stderr, retval = run_cmd(
+    auth_user,
+    PCS, 'qdevice', 'net-client', 'destroy'
+  )
+  if retval != 0
+    return [400, stderr.join('')]
+  end
+  return [200, stdout.join('')]
+end
+
+def qdevice_client_disable(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if disable_service('corosync-qdevice')
+    msg = 'corosync-qdevice disabled'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Disabling corosync-qdevice failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
+
+def qdevice_client_enable(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if not is_service_enabled?('corosync')
+    msg = 'corosync is not enabled, skipping'
+    $logger.info(msg)
+    return [200, msg]
+  elsif enable_service('corosync-qdevice')
+    msg = 'corosync-qdevice enabled'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Enabling corosync-qdevice failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
+
+def qdevice_client_stop(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if stop_service('corosync-qdevice')
+    msg = 'corosync-qdevice stopped'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Stopping corosync-qdevice failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
+
+def qdevice_client_start(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  if not is_service_running?('corosync')
+    msg = 'corosync is not running, skipping'
+    $logger.info(msg)
+    return [200, msg]
+  elsif start_service('corosync-qdevice')
+    msg = 'corosync-qdevice started'
+    $logger.info(msg)
+    return [200, msg]
+  else
+    msg = 'Starting corosync-qdevice failed'
+    $logger.error(msg)
+    return [400, msg]
+  end
+end
